/*-
 * Copyright (c) 2009 Michael Pounov <misho@aitbg.com>
 *   AITNET - Software & Network Solutions - All rights reserved.
 * Made for OpenFest 2009, lecture "Authentication gateways"
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: authpfmgr.c,v 1.1.1.1 2009/10/16 15:40:51 misho Exp $
 */

#include "global.h"
#include "dirs.h"
#include "files.h"


extern char compiled[];
extern char compiledby[];
extern char compilehost[];

struct tagArgs	args;

// --------------------------------

static void Logo()
{
	printf(" -= authpf installer =-  Installer for authentication gateway system\n"
			"  (C)`09 AITNET - Software & Network Solutions - <misho@aitbg.com>  \n"
			"=== Compiled at %s from %s@%s ===\n\n", compiled, compiledby, compilehost);
}

static void Usage()
{
	printf("Syntax: authpfmgr [option]\n"
			"  -i\t\tInstall authpf system [default]\n"
			"  -m <hostname>\tCustom system hostname [default=%s]\n"
			"  -g <gid>\tIf not find %s group, add thid GID [default=%hu]\n"
			"  -h\t\tHelp! This screen.\n\n", 
			args.szHostname, GRP_AUTHPF, GRP_GID);
}

// --------------------------------

int main(int argc, char **argv)
{
	char ch, *pos;
	struct group *grp;
	struct stat sb;
	FILE *f;

	args.nGID = GRP_GID;
	gethostname(args.szHostname, MAX_FQDN);
	Logo();
	while ((ch = getopt(argc, argv, "ihm:g:")) != -1)
		switch (ch) {
			case 'i':	// install *
				args.nMode |= MODE_INST;
				break;
			case 'm':	// hostname
				strlcpy(args.szHostname, optarg, MAX_FQDN);
				break;
			case 'g':
				args.nGID = (u_short) strtonum(optarg, 1, 65536, 
						(const char **) &pos);
				break;
			case 'h':	// help
			default:
				Usage();
				return 1;
		}
	argc -= optind;
	argv += optind;
	if (!args.nMode)
		args.nMode = MODE_INST;

	ch = 0;
	printf("RUN> Mode:: %s, Hostname:: %s\n\n", args.nMode == MODE_INST ? 
			"install" : "uninstall", args.szHostname);
	// 1 stage
	printf("(%d) Check for %s shell present ... ", ++ch, BIN_AUTHPF);
	if (stat(BIN_AUTHPF, &sb) == -1) {
		printf("error file #%d - %s\n", errno, strerror(errno));
		return 2;
	}
	if (sb.st_mode % 010000 != BIN_PERM) {
		printf("permission realy is %o but should be %o #%d - %s\n", 
				sb.st_mode % 010000, BIN_PERM, errno, strerror(errno));
		return 2;
	} else
		printf("#%o\n", BIN_PERM);
	// 2 stage
	grp = getgrnam(GRP_AUTHPF);
	printf("(%d) Check for %s group present ... #%d\n", ++ch, GRP_AUTHPF, !grp ? -1 : grp->gr_gid);
	if (!grp) {
		f = fopen("/etc/group", "a");
		if (!f) {
			printf("Error:: in open group file #%d - %s\n", errno, strerror(errno));
			return 3;
		}
		fprintf(f, "%s:*:%hu:\n", GRP_AUTHPF, args.nGID);
		fclose(f);

		grp = getgrnam(GRP_AUTHPF);
		printf("  + Check for %s group present ... #%d\n", GRP_AUTHPF, !grp ? -1 : grp->gr_gid);
		if (!grp)
			return 3;
	}
	// 3 stage
	if (CreateDirs(&ch))
		return 4;
	// 4 stage
	if (LoginClass(&ch) == -1)
		return 5;
	// 5 stage
	if (CopyFiles(&ch) == -1)
		return 6;
	// 6 stage
	if (fstabUpdate(&ch) == -1)
		return 7;
	// 7 stage
	if (rcUpdate(&ch))
		return 8;

	printf("\nDONE> Installation completed.\n");
	return 0;
}
