/*-
 * Copyright (c) 2009 Michael Pounov <misho@aitbg.com>
 *   AITNET - Software & Network Solutions - All rights reserved.
 * Made for OpenFest 2009, lecture "Authentication gateways"
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: files.c,v 1.1.1.1 2009/10/16 15:40:51 misho Exp $
 */

#include "global.h"


extern char authpf_message[];
extern char authpf_problem[];
extern char authpf_conf[];
extern char authpf_allow[];
extern char authpf_sshd[];


int LoginClass(char * __restrict ch)
{
	const char *csClass = "%s:\\\n"
		"\t:welcome=/etc/motd.authpf:shell=/usr/sbin/authpf:\\\n"
		"\t:tc=default:\n";
	char szClass[MAX_STR];
	login_cap_t *class, *chk;
	int res = -1;
	FILE *f;

	printf("(%d) Create login class %s ... ", ++*ch, CLASS_AUTHPF);
	class = login_getclassbyname(CLASS_AUTHPF, NULL);
	if (!class) {
		printf("error in %s !\n", CLASS_FILE);
		return res;
	}
	if (strcmp(class->lc_class, CLASS_AUTHPF)) {
		res = 0;
		memset(szClass, 0, MAX_STR);
		snprintf(szClass, MAX_STR, csClass, CLASS_AUTHPF);

		f = fopen(CLASS_FILE, "a");
		if (!f) {
			printf("error in %s #%d - %s\n", CLASS_FILE, errno, strerror(errno));
			res = -1;
			goto end;
		}
		fwrite(szClass, strlen(szClass), 1, f);
		fclose(f);

		memset(szClass, 0, MAX_STR);
		snprintf(szClass, MAX_STR, "%s %s", CLASS_MKDB, CLASS_FILE);
		switch (system(szClass)) {
			case 127:
				printf("error in shell!\n");
				res = -1;
				goto end;
			case -1:
				printf("error in fork!\n");
				res = -1;
				goto end;
			case 0:
				break;
			default:
				printf("error in make %s.db\n", CLASS_FILE);
				res = -1;
				goto end;
		}

		chk = login_getclassbyname(CLASS_AUTHPF, NULL);
		if (!chk) {
			printf("error in %s !\n", CLASS_FILE);
			res = -1;
			goto end;
		}
		if (strcmp(chk->lc_class, CLASS_AUTHPF)) {
			printf("error unable to create class !\n");
			login_close(chk);
			res = -1;
			goto end;
		}
		login_close(chk);
	} else
		res = 1;

	printf("#%d (%s)\n", res, !res ? CLASS_AUTHPF : class->lc_class);
end:
	login_close(class);
	return res;
}

int CopyFiles(char * __restrict ch)
{
	const char *Filez[] = { "/etc/authpf/authpf.allow", "/etc/authpf/authpf.conf", 
				"/etc/authpf/authpf.problem", "/etc/authpf/authpf.message", 
				"/etc/ssh/sshd_config_authpf", NULL };
	char *Bufz[] = { authpf_allow, authpf_conf, authpf_problem, authpf_message, authpf_sshd, NULL };
	register int i;
	FILE *f;
	int res;

	for (i = 0; Filez[i] && Bufz[i]; i++) {
		printf("(%d) Create file %s ... ", ++*ch, Filez[i]);
		f = fopen(Filez[i], "w");
		if (!f) {
			printf("error in create %s #%d - %s\n", Filez[i], errno, strerror(errno));
			return -1;
		}
		res = fwrite(Bufz[i], strlen(Bufz[i]), 1, f);
		fclose(f);
		printf("#%d\n", res);
	}

	return 0;
}

int fstabUpdate(char * __restrict ch)
{
	const char *csLine = "fdescfs\t\t/dev/fd\t\tfdescfs\t\trw\t\t0\t0\n";
	FILE *f;
	char szLine[MAX_STR], **app, *arr[7], *pos;
	int flg = -1;
	struct statfs fs;

	printf("(%d) Update %s ... ", ++*ch, FSTAB_FILE);
	f = fopen(FSTAB_FILE, "r+");
	if (!f) {
		printf("error in open %s #%d - %s\n", FSTAB_FILE, errno, strerror(errno));
		return flg;
	}

	while (!feof(f)) {
		memset(szLine, 0, MAX_STR);
		fgets(szLine, MAX_STR - 1, f);

		memset(arr, 0, sizeof arr);
		for (app = arr, pos = szLine; app < arr + 6 && (*app = strsep(&pos, " \t")); 
				**app ? app++ : app);

		if (*arr && !strcmp(arr[1], "/dev/fd") && !strcmp(arr[2], "fdescfs")) {
			flg = 0;
			break;
		}
	}

	if (flg) {
		fseek(f, 0, SEEK_END);
		flg = fwrite(csLine, strlen(csLine), 1, f);
	}

	printf("#%d\n", flg);
	fclose(f);

	printf("(%d) Check for already mounted fdescfs to /dev/fd ... ", ++*ch);
	if (statfs("/dev/fd", &fs) == -1) {
		printf("error statfs fdescfs #%d - %s\n", errno, strerror(errno));
		return -1;
	} else
		flg = !strcmp(fs.f_mntonname, "/dev/fd");
	printf("#%d (%s)\n", flg, fs.f_mntonname);

	if (!flg) {
		printf("(%d) Mount fdescfs to /dev/fd ... ", ++*ch);
		flg = mount("fdescfs", "/dev/fd", 0, "fdecsfs");
		if (flg == -1)
			printf("error mount fdescfs #%d - %s\n", errno, strerror(errno));
		else
			printf("#%d\n", flg);
	}

	return flg;
}

int rcUpdate(char * __restrict ch)
{
	const char *csLocal = "[ -x /etc/rc.authpf ] && /etc/rc.authpf";
	char szLine[MAX_STR];
	FILE *f;
	int flg = -1;

	printf("(%d) Update %s ... ", ++*ch, RC_LOCAL);
	f = fopen(RC_LOCAL, "r+");
	if (!f) {
		printf("error in open %s #%d - %s\n", RC_LOCAL, errno, strerror(errno));
		return flg;
	}

	while (!feof(f)) {
		memset(szLine, 0, MAX_STR);
		fgets(szLine, MAX_STR - 1, f);

		if (strstr(szLine, csLocal)) {
			flg = 0;
			break;
		}
	}

	if (flg) {
		fseek(f, 0, SEEK_END);
		fwrite("\n", 1, 1, f);
		flg = fwrite(csLocal, strlen(csLocal), 1, f);
		fwrite("\n", 1, 1, f);
	}

	printf("#%d\n", flg);
	fclose(f);
	return 0;
}
